//Code to readout the set-up registers
#include <Wire.h>
#include <dummy.h> 

//for esp32
extern TwoWire I2Cone= TwoWire(0);
extern TwoWire I2Ctwo= TwoWire(1);

#define FDC1 0x50
#define FDC2 0x50
#define NUM_SENSORS 4  
int OFFSET= 0; //in pF
int OFFSET2 = OFFSET;
double data_list1[6] = { };
double data_list2[6] = { };
int check1[2];
int check2[2];
int check_1[4];
int check_2[4];
int check1_new[2];
int check2_new[2];
int check_1_new;
int check_2_new;


void setup() {
  delay(1000); 
  Serial.print("Going through setup");
  Serial.print("");
  Serial.print("Going through setup");
  Serial.print("");
  Serial.print("Going through setup");
  Serial.print("");
  delay(1000); 
  int capdac = round(OFFSET/3.125);
  Serial.begin(115200);
  Serial.println("");
  I2Cone.begin(21, 22, 100000);// I2C pins on FDC1
  I2Ctwo.begin(33, 32, 100000); //I2C pins on FDC2

  // reset both FDCs 
  //to not having the flipping and the wrong registers set-up, I  reset twice the second FDC (pins 33,32) and once the first(21,22)
  // Nothing changes if I reset both FDC twice
  I2Cone.beginTransmission(FDC1);
  I2Cone.write(0x0C);
  I2Cone.write(0x80); 
  I2Cone.endTransmission();
  I2Ctwo.beginTransmission(FDC2);
  I2Ctwo.write(0x0C);
  I2Ctwo.write(0x80); 
  I2Ctwo.endTransmission();

  I2Cone.beginTransmission(FDC1);
  I2Cone.write(0x0C);
  I2Cone.write(0x80); 
  I2Cone.endTransmission();
  I2Ctwo.beginTransmission(FDC2);
  I2Ctwo.write(0x0C);
  I2Ctwo.write(0x80); 
  I2Ctwo.endTransmission();

  //Initiate CIN 1-4 for single ended measurement with offset of 15.625 pF
  for (int i = 0; i < NUM_SENSORS; i++) {  //Caculate register value from CAPDAC value 
    int register_value = 0x1000 + (capdac << 5);
    I2Cone.beginTransmission(FDC1);  
    I2Cone.write(0x08 + i); 
    I2Cone.write(highByte(register_value) + 32*i);
    I2Cone.write(lowByte(register_value));
    I2Cone.endTransmission();
    I2Ctwo.beginTransmission(FDC2);  
    I2Ctwo.write(0x08 + i); 
    I2Ctwo.write(highByte(register_value) + 32*i);
    I2Ctwo.write(lowByte(register_value));
    I2Ctwo.endTransmission();
  }
  //Initiate measurements with data rate of 100S/s and continuous measurements 
  I2Cone.beginTransmission(FDC1); 
  I2Cone.write(0x0C); //FDC Configuration Register 0x0C
  I2Cone.write(0x05); //sampling rate
  I2Cone.write(0xF0); 
  I2Cone.endTransmission();
  I2Ctwo.beginTransmission(FDC2); 
  I2Ctwo.write(0x0C); 
  I2Ctwo.write(0x05); 
  I2Ctwo.write(0xF0);  
  I2Ctwo.endTransmission();
  Serial.println("Process can start");

  //Readout the configurations registers to see if they were properly set up
  for (int i = 0; i < NUM_SENSORS; i++) {
    I2Cone.beginTransmission(FDC1); 
    I2Cone.write(0x08 + i); 
    I2Cone.endTransmission();
    I2Cone.requestFrom(FDC1, 2);
    for (int j = 0; j < 2; j++) {
      check1[j] = I2Cone.read();
    }
    check_1[i] = (check1[0] << 8) + check1[1];
  
    I2Ctwo.beginTransmission(FDC2); 
    I2Ctwo.write(0x08 + i); 
    I2Ctwo.endTransmission();
    I2Ctwo.requestFrom(FDC2, 2);
    for (int j = 0; j < 2; j++) {
      check2[j] = I2Ctwo.read();
    }
    check_2[i] = (check2[0] << 8) + check2[1];
  }
  //print out the values in binary
  for (int i = 0; i < NUM_SENSORS; i++) {
    int bitsCount = sizeof( check_1[i] ) * 8;
    char str[ bitsCount + 1 ];
    uint8_t j = 0;
    while ( bitsCount-- )
      str[ j++ ] = bitRead( check_1[i], bitsCount ) + '0';
    str[ j ] = '\0';
    Serial.print("Check1_" + String(i+1) + ":  ");
    Serial.println(str); // + (str, BIN) + " " );
  }
  Serial.println("");
  
  for (int i = 0; i < NUM_SENSORS; i++) {
    int bitsCount = sizeof( check_2[i] ) * 8;
    char str[ bitsCount + 1 ];
    uint8_t j = 0;
    while ( bitsCount-- )
      str[ j++ ] = bitRead( check_2[i], bitsCount ) + '0';
    str[ j ] = '\0';
    Serial.print("Check2_" + String(i+5) + ":  ");
    Serial.println(str); // + (str, BIN) + " " );
  } 
    Serial.println("");

  //rad again the first register for both FDC
  I2Cone.beginTransmission(FDC1); 
  I2Cone.write(0x08); 
  I2Cone.endTransmission();
  I2Cone.requestFrom(FDC1, 2);
  for (int j = 0; j < 2; j++) {
    check1_new[j] = I2Cone.read();
  }
  check_1_new = (check1_new[0] << 8) + check1_new[1];
  
  I2Ctwo.beginTransmission(FDC2); 
  I2Ctwo.write(0x08); 
  I2Ctwo.endTransmission();
  I2Ctwo.requestFrom(FDC2, 2);
  for (int j = 0; j < 2; j++) {
    check2_new[j] = I2Ctwo.read();
  }
  check_2_new = (check2_new[0] << 8) + check2_new[1];

  //print the new readback of the first register for both FDCs
  int bitsCount = sizeof( check_1_new ) * 8;
  char str[ bitsCount + 1 ];
  uint8_t j = 0;
  while ( bitsCount-- )
    str[ j++ ] = bitRead( check_1_new, bitsCount ) + '0';
  str[ j ] = '\0';
  Serial.print("Check1_new_" + String(1) + ":  ");
  Serial.println(str); // + (str, BIN) + " " );
  int bitsCount_n = sizeof( check_2_new) * 8;
  char str_n[ bitsCount_n + 1 ];
  uint8_t k = 0;
  while ( bitsCount_n-- )
    str_n[ k++ ] = bitRead( check_2_new, bitsCount_n ) + '0';
  str_n[ k ] = '\0';
  Serial.print("Check2_new_" + String(5) + ":  ");
  Serial.println(str_n); // + (str, BIN) + " " );
  

  //Calibration 
  Serial.println("");
  Serial.println("Calibration...");
  if (isReady1(0)) { 
    data_list1[0] = readC1(0); 
  }
  if (isReady1(1)) { 
    data_list1[1] = readC1(1); 
  }
  if (isReady1(2)) { 
    data_list1[2] = readC1(2); 
  }
  if (isReady1(3)) { 
    data_list1[3] = readC1(3); 
  }

  if (isReady2(0)) { 
    data_list2[0] = readC2(0); 
  }
  if (isReady2(1)) { 
    data_list2[1] = readC2(1); 
  }
  if (isReady2(2)) { 
    data_list2[2] = readC2(2); 
  }
  if (isReady2(3)) { 
    data_list2[3] = readC2(3); 
  }

  // during calibration the system waits until the channels are correclty read and associated to the sensors
  while (data_list1[0] == 0 || data_list1[1] == 0 || data_list1[2] == 0 || data_list1[3] == 0) { 
  // if either of the read values is 0, it means there is an error and I have to read again the whole set of measurements
    if (isReady1(0)) {
      data_list1[0] = readC1(0);
    }
    if (isReady1(1)) {
      data_list1[1] = readC1(1);
    }
    if (isReady1(2)) {
      data_list1[2] = readC1(2);
    }
    if (isReady1(3)) {
      data_list1[3] = readC1(3);
    }
  } 
  while (data_list2[0] == 0 || data_list2[1] == 0 || data_list2[2] == 0 || data_list2[3] == 0) { 
    if (isReady2(0)) {
      data_list2[0] = readC2(0);
    }
    if (isReady2(1)) {
      data_list2[1] = readC2(1);
    }
    if (isReady2(2)) {
      data_list2[2] = readC2(2);
    }
    if (isReady2(3)) {
      data_list2[3] = readC2(3);
    }
  } 
}


//loop runs continuously
void loop() {
//check if they are ready, read and print values --> here the printout is obmitted, only registers value are printed
  if (isReady1(0)) {
    data_list1[0] = readC1(0);
  }
  if (isReady1(1)) {
    data_list1[1] = readC1(1);
  }
  if (isReady1(2)) {
    data_list1[2] = readC1(2);
  }
  if (isReady1(3)) {
    data_list1[3] = readC1(3);
  }
  if (isReady2(0)) {
    data_list2[0] = readC2(0);
  }
  if (isReady2(1)) {
    data_list2[1] = readC2(1);
  }
  if (isReady2(2)) {
    data_list2[2] = readC2(2);
  }
  if (isReady2(3)) {
    data_list2[3] = readC2(3);
  }
  delay(1000); 
}

//Input: Number of input pin (starts at 0)
//Output: True if measurement at position cin_n is completed, false otherwise 
bool isReady1(int cin_n) {
  int fdcone[4];
  int i = 0;
  I2Cone.beginTransmission(FDC1); 
  I2Cone.write(0x0C);  
  I2Cone.endTransmission();
  I2Cone.requestFrom(FDC1, 4);
  while (I2Cone.available()) {
    fdcone[i] = I2Cone.read();  
    i++;
  }
  return bitRead(fdcone[1], 5 - cin_n);
}
bool isReady2(int cin_n) {
  int fdctwo[4];
  int i = 0;
  I2Ctwo.beginTransmission(FDC2); 
  I2Ctwo.write(0x0C); 
  I2Ctwo.endTransmission();
  I2Ctwo.requestFrom(FDC2, 4);
  while (I2Ctwo.available()) {
    fdctwo[i] = I2Ctwo.read(); 
    i++;
  }
  return bitRead(fdctwo[1], 5 - cin_n);
}

//Input: Number of input pin (starts at 0) //Output: -
//Prints capacitance at input cin_n to serial 
double readC1(int cin_n) {
  int32_t c;
  int c_data1[4]; 
  I2Cone.beginTransmission(FDC1); 
  I2Cone.write(0x00 + cin_n * 2); 
  I2Cone.endTransmission();
  I2Cone.requestFrom(FDC1, 4);
  //Read Measurement results on CINx
  for (int i = 0; i < 4; i++) { 
    c_data1[i] = I2Cone.read();
  }
  //Convert byte array to int and print capacitance to serial
  c = (c_data1[0] << 16) + (c_data1[1]<<8) + c_data1[2]; 
  //get the 2's ocmplement
  if (c & 0x800000) { // Check if value is negative
    c = ~c + 1; // Invert all bits and add 1
  }
  float cap1 = ((float)(c)/ 524288.0)+ OFFSET; 
  return cap1;
}

double readC2(int cin_n) {
  int32_t c;
  int c_data2[4]; 
  I2Ctwo.beginTransmission(FDC2); 
  I2Ctwo.write(0x00 + cin_n * 2); 
  I2Ctwo.endTransmission();
  I2Ctwo.requestFrom(FDC2, 4);
  //Read Measurement results on CINx
  for (int i = 0; i < 4; i++) { 
  c_data2[i] = I2Ctwo.read();
  }
  //Convert byte array to int and print capacitance to serial
  c = (c_data2[0] << 16) + (c_data2[1]<<8) + c_data2[2];
  //get the 2's complement
  if (c & 0x800000) { // Check if value is negative
    c = ~c + 1; // Invert all bits and add 1
  }
  float cap2 = ((float)(c)/ 524288.0)+ OFFSET; 
  return cap2;
}



